@@ -1,6 +1,6 @@
 import os
 
-from .base import Ostrich, Raven, get_average_annual_runoff
+from .base import Ostrich, Raven, RavenError, get_average_annual_runoff
 from .emulators import *
 from .multimodel import RavenMultiModel
 from .rv import HRU, LU, RV, RVI, HRUState, Sub
