@@ -12,7 +12,7 @@ but this is a good proof of concept.
 
 
 class TestHindcasting:
-    def test_hindcasting_GEPS(self, tmpdir):
+    def test_hindcasting_GEPS(self):
 
         # Prepare a RAVEN model run using historical data, GR4JCN in this case.
         # This is a dummy run to get initial states. In a real forecast situation,
@@ -20,15 +20,17 @@ class TestHindcasting:
         ts = get_local_testdata(
             "raven-gr4j-cemaneige/Salmon-River-Near-Prince-George_meteo_daily.nc"
         )
-        model = GR4JCN(workdir=tmpdir)
+        hrus = (
+            GR4JCN.LandHRU(
+                area=4250.6, elevation=843.0, latitude=54.4848, longitude=-123.3659
+            ),
+        )
+        model = GR4JCN()
         model(
             ts,
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2002, 6, 1),
-            area=44250.6,
-            elevation=843.0,
-            latitude=54.4848,
-            longitude=-123.3659,
+            hrus=hrus,
             params=(0.529, -3.396, 407.29, 1.072, 16.9, 0.947),
         )
 
@@ -52,10 +54,7 @@ class TestHindcasting:
             nc_index=range(nm),
             start_date=dt.datetime(2018, 6, 1),
             end_date=dt.datetime(2018, 6, 10),
-            area=44250.6,
-            elevation=843.0,
-            latitude=54.4848,
-            longitude=-123.3659,
+            hrus=hrus,
             params=(0.529, -3.396, 407.29, 1.072, 16.9, 0.947),
             overwrite=True,
             pr={
